import java.awt.geom.*;
import java.awt.*;
import java.util.*;

/**
 * Un componente electronico cualquiera, contiene todas sus representaciones
 * (graficas, electricas, etc.) 
 * @author Carlos Cortes
 */
public class Componente
{
    /**
     * Tipo de componente (Fuente, Resistencia, o Tierra).
     */
    private Modelo modelo;
    
    /**
     * Representacion grafica del componente.
     */
    private Figura figura;
    
    /**
     * @param tipo El tipo de componente, sus valores validos estan definidos
     * en Defs.java.
     * @param valor El valor del componente (volt, ohm, etc.)
     */
    public Componente(String tipo, float valor)
    {
        /* Creacion del tipo de componente que corresponda */
        if (tipo.equals(Defs.FUENTECC))
        {
            modelo = new FuenteCC(valor);
            figura = new FigFuenteCC();
        }
        else
        if (tipo.equals(Defs.RESISTOR))
        {
            modelo = new Resistor(valor);
            figura = new FigResistor();
        }
        else
        if (tipo.equals(Defs.GND))
        {
            modelo = new GND();
            figura = new FigGND();
        }
        else /* En caso de un 'tipo' desconocido */
        {
            System.out.println("Error: Tipo de componente desconocido\n    "
                               + tipo);
            System.exit(1);
        }
    }
    
    /**
     * @return La posicion del punto central de la figura que corresponde
     * al componente.
     */
    public Point getCentro()
    {
        return figura.getCentro();
    }
    
    /**
     * @return La lista de los pines de la figura que corresponde
     * al componente.
     */
    public ArrayList < Point > getPinList()
    {
        return figura.getPinList();
    }
    
    /**
     * @return Indica si el componente tiene algun valor para mostrar en
     * forma de texto.
     */
    public boolean tieneValor()
    {
        return !(modelo instanceof GND);
    }
    
    /**
     * @return El valor (voltaje o resistencia) del componente
     */
    public float getValor()
    {
        return modelo.getValor();
    }
    
    /**
     * @return La posicion del cuadro de texto de la figura
     * que corresponde al componente.
     */
    public Point getPosTexto()
    {
        return figura.getPosTexto();
    }
    
    /**
     * @return La cantidad de elementos (Shapes) que componen la figura,
     * que es la representacion grafica del componente.
     */
    public int numShapes()
    {
        return figura.numShapes();
    }
    
    /**
     * @param n Indice del elemento (Shape) que se quiere
     * obtener de la lista figList.
     * @return El elemento (Shape) 'n', de los que componen la figura
     * que corresponde al componente.
     */
    public Shape getShape(int n)
    {
        return figura.getShape(n);
    }
    
    /**
     * @return Un Rectangulo que define el area activa de la figura
     * que corresponde al componente.
     */
    public Rectangle getBounds()
    {
        return figura.getBounds();
    }
        
    /**
     * Rota el componente en 90 grados (sentido reloj) respecto
     * al punto central.
     */
    public void rotar()
    {
        figura.rotar();
    }
    
    /**
     * Traslada el componente al punto (x,y).
     * @param x nueva coordenada 'x'.
     * @param y nueva coordenada 'y'.
     */
    public void trasladar(int x, int y)
    {
        figura.trasladar(x, y);
    }
    
    /**
     * Conecta el componete con otro
     * @param pp pin de este 'this' componente
     * @param otro el otro componente
     * @param po pin del otro componente
     */
    public void conectarCon(int pp, Componente otro, int po)
    {
        int x, y;
        
        x = (int)(otro.getPinList().get(po).getX()
            - this.getPinList().get(pp).getX()
            + this.getCentro().getX());
        y = (int)(otro.getPinList().get(po).getY()
            - this.getPinList().get(pp).getY()
            + this.getCentro().getY());
        
        this.trasladar(x, y);
    }
}
